function out_sig = perform_psola(frames, f0_factor)

% out_sig = perform_psola(frames, f0_factor)
% 
% This function performs the "Pitch Synchronous Overlal and Add algorithm" 
% to construct the output signal using the input cell array of short-term
% frames and f0_factor.
% 
% frames        - input short-term time-domain frame cell array
% f0_factor     - overlap parameter
% out_sig       - output signal

%% Paths and variables
if ((nargin < 2) || isempty(f0_factor))
    f0_factor = 1;
end

%% Prepare the output signal
cur_pos = 1;
org_pos = 0;
dif_pos = 0;

for i = 1:(length(frames) - 2)
    org_len = length(frames{i}) + length(frames{i + 1});
    new_len = round(org_len*f0_factor);
    
    if (f0_factor > 1.0)
        if (org_pos + new_len >= cur_pos);
            i = i + 1;
            org_len = length(frames{i}) + length(frames{i + 1});
            new_len = round(org_len*f0_factor);
        end
    end
    
    cur_pos = cur_pos + round(length(frames{i})*f0_factor);
    dif_pos = dif_pos + abs(new_len - org_len);
    org_pos = org_pos + org_len;
    
    if (f0_factor < 1.0)
        if (dif_pos >= new_len)
            cur_pos = cur_pos + round(length(frames{i})*f0_factor);
            dif_pos = dif_pos + abs(new_len - org_len) - new_len;
        end
    end
end

%% Construct the output signal
out_sig = zeros(cur_pos + org_len + new_len, 1);
cur_pos = 1;
org_pos = 0;
dif_pos = 0;

for i = 1:(length(frames) - 2)    
    org_len = length(frames{i}) + length(frames{i + 1});
    new_len = round(org_len*f0_factor);
    
    if (f0_factor > 1.0)
        if (org_pos + new_len >= cur_pos)
            i = i + 1;
            org_len = length(frames{i}) + length(frames{i + 1});
            new_len = round(org_len*f0_factor);
        end
    end
    
    out_sig(cur_pos:(cur_pos + length(hanning(new_len)) - 1)) = ...
        out_sig(cur_pos:(cur_pos + length(hanning(new_len)) - 1)) + ...
        interp1(1:org_len, ...
            [frames{i}; frames{i + 1}], ...
            1:new_len, ...
            'linear', ...
            'extrap' ...
        )'.*hanning(new_len);
        
    cur_pos = cur_pos + round(length(frames{i})*f0_factor);
    dif_pos = dif_pos + abs(new_len - org_len);
    org_pos = org_pos + org_len;
    
    if (f0_factor < 1.0)
        if (dif_pos >= new_len)
            pos_str = cur_pos;
            pos_end = (cur_pos + length(hanning(new_len)) - 1);
            
            out_sig(pos_str:pos_end) = ...
                out_sig(pos_str:pos_end) + ...
                interp1(1:org_len, ...
                    [frames{i}; frames{i + 1}], ...
                    1:new_len ...
                )'.*hanning(new_len);
           
            cur_pos = cur_pos + round(length(frames{i})*f0_factor);
            dif_pos = dif_pos + abs(new_len - org_len) - new_len;
        end
    end
end