function warped_freqs = perform_vtln(frames, warp_function, alpha)

% warped_freqs = perform_vtln(frames, warp_function, alpha)
% 
% This function performs frequency warping using predefined set of warping
% functions and an input warping parameter alpha and returns output cell 
% array of warped frames.
% 
% frames            - input short-term frequency-domain frame cell array
% warp_function     - frequency warping function (default: 'bilinear')
% alpha             - parameter of frequency warping (default: 0.1235)
% warped_freqs      - output short-term frequency-domain frame cell array
% 
% Supported frequency warping functions:
%  - 'asymmetric'
%  - 'symmetric'
%  - 'power'
%  - 'quadratic'
%  - 'bilinear' (default)

%% Paths and variables
if ((nargin < 2) || isempty(warp_function))
    warp_function = 'bilinear';
end
if ((nargin < 3) || isempty(alpha))
    alpha = 0.1235;
end

%% Check the frequency warping function
if ~(strcmpi(warp_function, 'asymmetric') || ...
    strcmpi(warp_function, 'symmetric')   || ...
    strcmpi(warp_function, 'power')       || ...
    strcmpi(warp_function, 'quadratic')   || ...
    strcmpi(warp_function, 'bilinear'))
    error('Invalid frequency warping function selected');
end

%% Prepare variables
if (strcmpi(warp_function, 'asymmetric'))
    omega0 = 7/8*pi;
end
if (strcmpi(warp_function, 'symmetric'))
    if (alpha > 1)
        omega0 = 7/(8*alpha)*pi;
    else
        omega0 = 7/8*pi;
    end
end

%% Prepare the output cell array
warped_freqs = cell(1, length(frames));

%% Warp the frames' frequeny representation
for i = 1:length(frames)
    
    % Prepare the warping variables
    frame = frames{i};
    omega = (1:length(frame))./length(frame).* pi;
    omega_warped = omega;
    
    % Asymmetric frequency warping
    if (strcmpi(warp_function, 'asymmetric'))
        omega_warped(omega<=omega0) = alpha.*omega(omega<=omega0);
        omega_warped(omega>omega0)  = alpha*omega0 + ...
            ((pi-alpha*omega0)/(pi-omega0)).*(omega(omega>omega0)-omega0);
        omega_warped(omega_warped>=pi) = pi-0.00001+ ...
            0.00001*(omega_warped(omega_warped>=pi));
    
    % Symmetric frequency warping
    elseif (strcmpi(warp_function, 'symmetric'))
        omega_warped(omega<=omega0) = alpha.*omega(omega<=omega0);
        omega_warped(omega>omega0)  = alpha*omega0 + ...
            ((pi-alpha*omega0)/(pi-omega0)).*(omega(omega>omega0)-omega0);
        omega_warped(omega_warped>=pi) = pi-0.00001+ ...
            0.00001*(omega_warped(omega_warped>=pi));
        
    % Power frequency warping
    elseif (strcmp(warp_function, 'power'))
        omega_warped = pi.*(omega./pi).^alpha;
        
    % Quadratic frequency warping
    elseif (strcmp(warp_function, 'quadratic'))
        omega_warped = omega+alpha.*(omega./pi-(omega./pi).^2);
        
    % Bilinear frequency warping
    elseif (strcmp(warp_function, 'bilinear'))
        z = exp(omega.*1i);
        omega_warped = abs(-1i.*log((z-alpha)./(1-alpha.*z)));
    end

    % Post-process the warped frequency spectrum
    omega_warped = omega_warped./pi.*length(frame);
    warped_frame = interp1( ...
        (1:length(frame)),  ...
        frames{i},          ...
        omega_warped,       ...
        'linear'            ...
    ).';
    
    if (isreal(frames{i}(end)))
        warped_frame(end) = real(warped_frame(end));
    end
    
    warped_frame(isnan(warped_frame)) = 0;
    warped_freqs{i} = warped_frame;
end