function z = vocal_tract_length_normalization(y, fs, warp_function, alpha)

% z = vocal_tract_length_normalization(y, fs, warp_function, alpha)
% 
% This function performs the vocal tratc length normalization using the
% find_pmarks algorithm written by W. Goncharoff (goncharo@ece.uic.edu).
% 
% y                 - input column vetcor of speech
% fs                - sampling frequency [Hz]
% warp_function     - frequency warping function (default: 'bilinear')
% alpha             - parameter of frequency warping (default: 0.1235)
% z                 - output column vector with speech
% 
% Supported frequency warping functions:
%  - 'asymmetric'
%  - 'symmetric' 
%  - 'power'
%  - 'quadratic'
%  - 'bilinear' (default)

%% Paths and variables
if ((nargin < 3) || isempty(warp_function))
    warp_function = 'bilinear';
end
if ((nargin < 4) || isempty(alpha))
    alpha = 0.1235;
end

%% Normalize the speech
y = y./max(abs(y));

%% Split the signal into the frames and calculate spectrum
frames = perform_synchronized_segmentation(y, fs);

%% Compute the spectrum
spectrum = convert_time2freq(frames);

%% Warp the spectrum
warped_spectrum = perform_vtln(spectrum, warp_function, alpha);

%% Convert spectrum into time and perform PSOLA
warped_signal = convert_freq2time(warped_spectrum);

%% Perform PSOLA
z = perform_psola(warped_signal, 1);